@@ -24,6 +24,7 @@ import scaffold
 from test_pidlockfile import (
     FakeFileDescriptorStringIO,
     setup_pidfile_fixtures,
+    make_pidlockfile_scenarios,
     )
 from test_daemon import (
     setup_streams_fixtures,
@@ -34,11 +35,44 @@ from daemon import pidlockfile
 from daemon import runner
 
 
+class Exception_TestCase(scaffold.Exception_TestCase):
+    """ Test cases for module exception classes. """
+
+    def __init__(self, *args, **kwargs):
+        """ Set up a new instance. """
+        super(Exception_TestCase, self).__init__(*args, **kwargs)
+
+        self.valid_exceptions = {
+            runner.DaemonRunnerError: dict(
+                min_args = 1,
+                types = (Exception,),
+                ),
+            runner.DaemonRunnerInvalidActionError: dict(
+                min_args = 1,
+                types = (runner.DaemonRunnerError, ValueError),
+                ),
+            runner.DaemonRunnerStartFailureError: dict(
+                min_args = 1,
+                types = (runner.DaemonRunnerError, RuntimeError),
+                ),
+            runner.DaemonRunnerStopFailureError: dict(
+                min_args = 1,
+                types = (runner.DaemonRunnerError, RuntimeError),
+                ),
+            }
+
+
+def set_pidlockfile_scenario(testcase, scenario_name):
+    """ Set the PIDLockFile test scenario for the test case. """
+    scenarios = testcase.pidlockfile_scenarios
+    testcase.pidlockfile_scenario = scenarios[scenario_name]
+
 def setup_runner_fixtures(testcase):
     """ Set up common test fixtures for DaemonRunner test case. """
     testcase.mock_tracker = scaffold.MockTracker()
 
     setup_pidfile_fixtures(testcase)
+    testcase.pidlockfile_scenarios = make_pidlockfile_scenarios()
 
     setup_streams_fixtures(testcase)
 
@@ -48,12 +82,12 @@ def setup_runner_fixtures(testcase):
         mock_obj=testcase.mock_stderr,
         tracker=testcase.mock_tracker)
 
-    testcase.mock_pidfile_path = tempfile.mktemp()
+    set_pidlockfile_scenario(testcase, 'not-exist')
 
     testcase.mock_pidlockfile = scaffold.Mock(
         "pidlockfile.PIDLockFile",
         tracker=testcase.mock_tracker)
-    testcase.mock_pidlockfile.path = testcase.mock_pidfile_path
+    testcase.mock_pidlockfile.path = testcase.pidlockfile_scenario['path']
 
     scaffold.mock(
         "pidlockfile.PIDLockFile",
@@ -66,7 +100,7 @@ def setup_runner_fixtures(testcase):
             self.stdin_path = testcase.stream_file_paths['stdin']
             self.stdout_path = testcase.stream_file_paths['stdout']
             self.stderr_path = testcase.stream_file_paths['stderr']
-            self.pidfile_path = testcase.mock_pidfile_path
+            self.pidfile_path = testcase.pidlockfile_scenario['path']
 
         run = scaffold.Mock(
             "TestApp.run",
@@ -152,6 +186,14 @@ class DaemonRunner_TestCase(scaffold.TestCase):
         """ Should have specified application object. """
         self.failUnlessIs(self.test_app, self.test_instance.app)
 
+    def test_sets_pidfile_none_when_pidfile_path_is_none(self):
+        """ Should set ‘pidfile’ to ‘None’ when ‘pidfile_path’ is ‘None’. """
+        pidfile_path = None
+        self.test_app.pidfile_path = pidfile_path
+        expect_pidfile = None
+        instance = runner.DaemonRunner(self.test_app)
+        self.failUnlessIs(expect_pidfile, instance.pidfile)
+
     def test_error_when_pidfile_path_not_string(self):
         """ Should raise ValueError when PID file path not a string. """
         pidfile_path = object()
@@ -172,7 +214,7 @@ class DaemonRunner_TestCase(scaffold.TestCase):
 
     def test_creates_pidlockfile(self):
         """ Should create a PIDLockFile with the specified PID file path. """
-        pidfile_path = self.mock_pidfile_path
+        pidfile_path = self.pidlockfile_scenario['path']
         expect_mock_output = """\
             ...
             Called pidlockfile.PIDLockFile(%(pidfile_path)r)
@@ -351,10 +393,10 @@ class DaemonRunner_do_action_TestCase(scaffold.TestCase):
         scaffold.mock_restore()
 
     def test_raises_error_if_unknown_action(self):
-        """ Should emit a usage message and exit if too few arguments. """
+        """ Should emit a usage message and exit if action is unknown. """
         instance = self.test_instance
         instance.action = 'bogus'
-        expect_error = ValueError
+        expect_error = runner.DaemonRunnerInvalidActionError
         self.failUnlessRaises(
             expect_error,
             instance.do_action)
@@ -370,6 +412,7 @@ class DaemonRunner_do_action_start_TestCase(scaffold.TestCase):
 
         self.test_instance.action = 'start'
 
+        set_pidlockfile_scenario(self, 'not-exist')
         self.mock_pidlockfile.is_locked.mock_returns = False
         self.mock_pidlockfile.i_am_locking.mock_returns = False
         self.mock_pidlockfile.read_pid.mock_returns = None
@@ -378,14 +421,17 @@ class DaemonRunner_do_action_start_TestCase(scaffold.TestCase):
         """ Tear down test fixtures. """
         scaffold.mock_restore()
 
-    def test_aborts_if_pidfile_locked(self):
-        """ Should raise SystemExit if PID file is locked. """
+    def test_raises_error_if_pidfile_locked(self):
+        """ Should raise error if PID file is locked. """
+        set_pidlockfile_scenario(self, 'exist-other-pid-locked')
         instance = self.test_instance
         self.mock_pidlockfile.is_locked.mock_returns = True
         self.mock_pidlockfile.i_am_locking.mock_returns = False
-        self.mock_pidlockfile.read_pid.mock_returns = self.mock_other_pid
-        pidfile_path = self.mock_pidfile_path
-        expect_error = SystemExit
+        self.mock_pidlockfile.read_pid.mock_returns = (
+            self.pidlockfile_scenario['pidfile_pid'])
+        pidfile_path = self.pidlockfile_scenario['path']
+        expect_error = runner.DaemonRunnerStartFailureError
+        expect_message_content = pidfile_path
         try:
             instance.do_action()
         except expect_error, exc:
@@ -393,16 +439,18 @@ class DaemonRunner_do_action_start_TestCase(scaffold.TestCase):
         else:
             raise self.failureException(
                 "Failed to raise " + expect_error.__name__)
-        self.failUnlessIn(exc.message, pidfile_path)
+        self.failUnlessIn(exc.message, expect_message_content)
 
     def test_breaks_lock_if_no_such_process(self):
         """ Should request breaking lock if PID file process is not running. """
+        set_pidlockfile_scenario(self, 'exist-other-pid-locked')
         instance = self.test_instance
         self.mock_pidlockfile.is_locked.mock_returns = True
         self.mock_pidlockfile.i_am_locking.mock_returns = False
-        self.mock_pidlockfile.read_pid.mock_returns = self.mock_other_pid
-        pidfile_path = self.mock_pidfile_path
-        test_pid = self.mock_other_pid
+        self.mock_pidlockfile.read_pid.mock_returns = (
+            self.pidlockfile_scenario['pidfile_pid'])
+        pidfile_path = self.pidlockfile_scenario['path']
+        test_pid = self.pidlockfile_scenario['pidfile_pid']
         expect_signal = signal.SIG_DFL
         error = OSError(errno.ESRCH, "Not running")
         os.kill.mock_raises = error
@@ -430,7 +478,7 @@ class DaemonRunner_do_action_start_TestCase(scaffold.TestCase):
     def test_emits_start_message_to_stderr(self):
         """ Should emit start message to stderr. """
         instance = self.test_instance
-        current_pid = self.mock_current_pid
+        current_pid = self.pidlockfile_scenario['pid']
         expect_stderr = """\
             started with pid %(current_pid)d
             """ % vars()
@@ -459,22 +507,27 @@ class DaemonRunner_do_action_stop_TestCase(scaffold.TestCase):
 
         self.test_instance.action = 'stop'
 
+        set_pidlockfile_scenario(self, 'exist-other-pid-locked')
         self.mock_pidlockfile.is_locked.mock_returns = True
         self.mock_pidlockfile.i_am_locking.mock_returns = False
-        self.mock_pidlockfile.read_pid.mock_returns = self.mock_other_pid
+        self.mock_pidlockfile.read_pid.mock_returns = (
+            self.pidlockfile_scenario['pidfile_pid'])
 
     def tearDown(self):
         """ Tear down test fixtures. """
         scaffold.mock_restore()
 
-    def test_aborts_if_pidfile_not_locked(self):
-        """ Should report failure and exit if PID file is not locked. """
+    def test_raises_error_if_pidfile_not_locked(self):
+        """ Should raise error if PID file is not locked. """
+        set_pidlockfile_scenario(self, 'not-exist')
         instance = self.test_instance
         self.mock_pidlockfile.is_locked.mock_returns = False
         self.mock_pidlockfile.i_am_locking.mock_returns = False
-        self.mock_pidlockfile.read_pid.mock_returns = None
-        pidfile_path = self.mock_pidfile_path
-        expect_error = SystemExit
+        self.mock_pidlockfile.read_pid.mock_returns = (
+            self.pidlockfile_scenario['pidfile_pid'])
+        pidfile_path = self.pidlockfile_scenario['path']
+        expect_error = runner.DaemonRunnerStopFailureError
+        expect_message_content = pidfile_path
         try:
             instance.do_action()
         except expect_error, exc:
@@ -483,19 +536,18 @@ class DaemonRunner_do_action_stop_TestCase(scaffold.TestCase):
             raise self.failureException(
                 "Failed to raise " + expect_error.__name__)
         scaffold.mock_restore()
-        self.failUnlessIn(exc.message, pidfile_path)
+        self.failUnlessIn(exc.message, expect_message_content)
 
-    def test_breaks_lock_if_pidfile_lock_stale(self):
-        """ Should break lock if PID file lock is stale. """
+    def test_breaks_lock_if_pidfile_stale(self):
+        """ Should break lock if PID file is stale. """
         instance = self.test_instance
-        pidfile_path = self.mock_pidfile_path
-        test_pid = self.mock_other_pid
+        pidfile_path = self.pidlockfile_scenario['path']
+        test_pid = self.pidlockfile_scenario['pidfile_pid']
         expect_signal = signal.SIG_DFL
         error = OSError(errno.ESRCH, "Not running")
         os.kill.mock_raises = error
         expect_mock_output = """\
             ...
-            Called os.kill(%(test_pid)r, %(expect_signal)r)
             Called pidlockfile.PIDLockFile.break_lock()
             """ % vars()
         instance.do_action()
@@ -505,7 +557,7 @@ class DaemonRunner_do_action_stop_TestCase(scaffold.TestCase):
     def test_sends_terminate_signal_to_process_from_pidfile(self):
         """ Should send SIGTERM to the daemon process. """
         instance = self.test_instance
-        test_pid = self.mock_other_pid
+        test_pid = self.pidlockfile_scenario['pidfile_pid']
         expect_signal = signal.SIGTERM
         expect_mock_output = """\
             ...
@@ -515,17 +567,24 @@ class DaemonRunner_do_action_stop_TestCase(scaffold.TestCase):
         scaffold.mock_restore()
         self.failUnlessMockCheckerMatch(expect_mock_output)
 
-    def test_aborts_if_cannot_send_signal_to_process(self):
-        """ Should raise SystemExit if cannot send signal to daemon process. """
+    def test_raises_error_if_cannot_send_signal_to_process(self):
+        """ Should raise error if cannot send signal to daemon process. """
         instance = self.test_instance
-        test_pid = self.mock_other_pid
-        pidfile_path = self.mock_pidfile_path
+        test_pid = self.pidlockfile_scenario['pidfile_pid']
+        pidfile_path = self.pidlockfile_scenario['path']
         error = OSError(errno.EPERM, "Nice try")
         os.kill.mock_raises = error
-        expect_error = SystemExit
-        self.failUnlessRaises(
-            expect_error,
-            instance.do_action)
+        expect_error = runner.DaemonRunnerStopFailureError
+        expect_message_content = str(test_pid)
+        try:
+            instance.do_action()
+        except expect_error, exc:
+            pass
+        else:
+            raise self.failureException(
+                "Failed to raise " + expect_error.__name__)
+        scaffold.mock_restore()
+        self.failUnlessIn(exc.message, expect_message_content)
 
 
 class DaemonRunner_do_action_restart_TestCase(scaffold.TestCase):
